import tkinter  as tk
import tkinter.ttk as ttk
from customtkinter import *
from PIL import Image, ImageTk
from ttkwidgets import tooltips
import customtkinter
import screeninfo
import pollenisatorgui.core.components.utils as utils
import os
import json


def craftMenuWithStyle(menu_parent):
    """
    Craft a menu with the current style.
    Args:
        menu_parent: the parent menu to craft.

    Returns:
        the crafted menu.
    """
   
    menu = tk.Menu(menu_parent, tearoff=0, background= getStrongColor(), foreground="white", activebackground= getStrongActiveColor(), activeforeground="white")
    return menu

def setStyle(tkApp, dark_mode=False, _event=None):
    """
    Set the tk app style window widget style using ttk.Style
    Args:
        _event: not used but mandatory
    """
    if dark_mode:
        customtkinter.set_appearance_mode("dark")
    else:
        customtkinter.set_appearance_mode("light")

    style = ttk.Style(tkApp)
    style.theme_use("clam")
    
    colors = get_color_scheme(dark_mode)
    main_color = colors.get("main")
    hover_color = colors.get("hover") 
    background_color = colors.get("background")
    text_color = colors.get("text")
    background_bis_color = colors.get("background_bis")
    style.configure("Treeview.Heading", background=main_color,
                foreground="gray97", borderwidth=0)
    style.map('Treeview.Heading', background=[('active', hover_color)])
    style.configure("Treeview", background=background_color, fieldbackground=background_color, foreground=text_color)
    #style.configure("Treeview.Item")

    style.configure("TLabelframe", background=background_color,foreground=background_color,
                    labeloutside=False, bordercolor=main_color,relief=tk.SUNKEN)
    style.configure('TLabelframe.Label', background=main_color,
                    foreground=background_color, font=('Sans', '10', 'bold'))
    style.configure("TProgressbar",
                background=hover_color, foreground=hover_color, troughcolor=background_color, darkcolor=hover_color, lightcolor=hover_color)
    style.configure("Important.TFrame", background=main_color)
    style.configure("TFrame", background=background_color)
    style.configure("Debug.TFrame", background="red")
    style.configure("Important.TLabel", background=main_color, foreground=background_color)
    style.configure("Pagination.TLabel", background=background_color, foreground=main_color, font=('Sans', '10', 'underline') )
    style.configure("CurrentPagination.TLabel", background=background_color, foreground=main_color, font=('Sans', '10', 'bold') )
    style.map("Pagination.TLabel", foreground=[("active", hover_color)])
    style.map("CurrentPagination.TLabel", foreground=[("active", hover_color)] )
    style.configure("TLabel", background=background_color, foreground=text_color)
    style.configure("TCombobox", background=background_color)
    
    style.configure("TCheckbutton", background=background_color)

    style.configure("TButton", background=main_color,
                foreground=text_color, font=('Sans', '10', 'bold'), borderwidth=1)
    style.configure("icon.TButton", background=background_color, borderwidth=0)
    style.configure("iconbis.TButton", background=background_bis_color, borderwidth=0)
    style.configure("icon_white.TButton", background=main_color, borderwidth=0)

    style.configure("link.TButton", background=background_color, foreground=main_color, font=('Sans', '10', 'underline'), borderwidth=0)
    style.map('link.TButton', foreground=[('active', hover_color)], background=[('active', background_color)])
    style.configure("Notebook.TButton", background=main_color,
                foreground=text_color, font=('Sans', '10', 'bold'), borderwidth=0)
    style.configure("Notebook.TFrame", background=main_color)
    style.map('TButton', background=[('active', hover_color)])
    #  FIX tkinter tag_configure not showing colors   https://bugs.python.org/issue36468
    style.map('Treeview', foreground=fixedMap('foreground', style),
                background=fixedMap('background', style))
    tooltips.update_defaults({"background": getBackgroundColor()})

def getBackgroundColor():
    """
    Return the background color of the current style.
    Returns:
        the background color.
    """
    settings = utils.cacheSettings()
    dark_mode = settings.is_dark_mode()
    colors = get_color_scheme(dark_mode)
    return colors.get("background")

def getBackgroundSecondColor():
    settings = utils.cacheSettings()
    dark_mode = settings.is_dark_mode()
    colors = get_color_scheme(dark_mode)
    return colors.get("background_bis")

def getTextColor():
    """
    Return the background color of the current style.
    Returns:
        the background color.
    """
    settings = utils.cacheSettings()
    dark_mode = settings.is_dark_mode()
    colors = get_color_scheme(dark_mode)
    return colors.get("text")

def getStrongColor():
    """
    Return the background color of the current style.
    Returns:
        the background color.
    """
    settings = utils.cacheSettings()
    dark_mode = settings.is_dark_mode()
    colors = get_color_scheme(dark_mode)
    return colors.get("strong")

def getStrongActiveColor():
    """
    Return the background color of the current style.
    Returns:
        the background color.
    """
    settings = utils.cacheSettings()
    dark_mode = settings.is_dark_mode()
    colors = get_color_scheme(dark_mode)
    return colors.get("strong_active")

def get_color_scheme(dark_mode=False):
    main_color = ("#5ac433", "#2FA572")[dark_mode]
    hover_color = ("#48a825", "#2FC572")[dark_mode]
    background_color = ("gray97", "gray17")[dark_mode]
    text_color = ("gray20", "#DCE4EE")[dark_mode]
    second_color = ("gray89", "dim gray")[dark_mode]
    strong_color = ('#3b75a8','#113759')[dark_mode]
    strong_active_color = ('#061b4e','#2768a1')[dark_mode]
    return {"main":main_color, "hover":hover_color, "background":background_color, "text":text_color, "background_bis":second_color,
            "strong":strong_color, "strong_active":strong_active_color}


def fixedMap(option, style):
    """
    Fix color tag in treeview not appearing under some linux distros
    Args:
        option: the string option you want to affect on treeview ("background" for example)
        strle: the style object of ttk
    """
    # Fix for setting text colour for Tkinter 8.6.9
    # From: https://core.tcl.tk/tk/info/509cafafae
    #  FIX tkinter tag_configure not showing colors   https://bugs.python.org/issue36468
    # Returns the style map for 'option' with any styles starting with
    # ('!disabled', '!selected', ...) filtered out.

    # style.map() returns an empty list for missing options, so this
    # should be future-safe.
    return [elm for elm in style.map('Treeview', query_opt=option) if
            elm[:2] != ('!disabled', '!selected')]
                  



def get_screen_where_widget(widget):
    """
    Get the screen size of the current terminal

    Returns:
        Return the screen size as a tuple (width, height)
    """
    monitors = screeninfo.get_monitors()
    monitor = None
    if widget is not None:
        x = widget.winfo_rootx()
        y = widget.winfo_rooty()
        for m in reversed(monitors):
            if m.x <= x <= m.width + m.x and m.y <= y <= m.height + m.y:
                monitor = m
    if monitor is None:
        monitor = monitors[0]
    return monitor



def getValidMarkIconPath():
    """Returns:
         a validation mark icon path
    """
    return getIcon("done_tool.png")


def getBadMarkIconPath():
    """Returns:
         a bad mark icon path
    """
    return getIcon("cross.png")


def getWaitingMarkIconPath():
    """Returns:
         a waiting icon path
    """
    return getIcon("waiting.png")

def getIcon(name):
    """Returns : the path to an specified icon name
    """
    settings = utils.cacheSettings()
    if settings.is_dark_mode():
        return os.path.join(getIconDir(), "dark", os.path.basename(name))
    else:
        return os.path.join(getIconDir(), "light", os.path.basename(name))
        
def getHelpIconPath():
    """Returns:
         a help icon path
    """
    return getIcon("help.png")

def loadIcon(name, **kwargs):
    """Returns:
         a help icon path
    """
    path = getIcon(name)
    image = Image.open(path)
    #img = image.resize((16, 16))
    return ImageTk.PhotoImage(image)

def loadIcons(names):
    load_icons = {}
    for name in names:
        load_icons[name] = loadIcon(name)
    return load_icons

def getIconDir():
    """Returns:
        the icon directory path
    """
    p = os.path.join(os.path.dirname(
        os.path.realpath(__file__)), "../../theme/")
    return p

def getColorTheme():
    return utils.getMainDir()+"theme/pollenisator_theme.json"

def loadTheme():
    with open(getColorTheme(), "r") as f:
        return json.load(f)


import json
from datetime import datetime
from pprint import pprint
from bson import ObjectId

from pollenisatorgui.core.components.apiclient import APIClient
from pollenisatorgui.pollenisator import consoleConnect
    
# Create a new worksheet using openpyxl
from openpyxl import Workbook
from openpyxl.styles import PatternFill, Font, Alignment
from openpyxl.utils import get_column_letter


def testsslresults(apiclient):
    is_sncf: bool = False
    client_settings = apiclient.findInDb(apiclient.getCurrentPentest(), "settings", {"key":"client_name"}, multi=False)
    if client_settings is not None:
        is_sncf = "sncf" in client_settings.get("value", "").lower()
    tags_data = apiclient.findInDb(apiclient.getCurrentPentest(), "tags", { "tags.name": "SSL/TLS-flaws" }, multi=True)
    port_ids = []
    if tags_data is not None:
        for tag in tags_data:
            if tag.get("item_type","") == "ports":
                port_ids.append(tag.get("item_id",""))
    expectedVulns = ["SSLv2","SSLv3","TLS1","TLS1_1","TLS1_2","TLS1_3","FS","cipherlist_NULL","cipherlist_aNULL","cipherlist_EXPORT","cipherlist_DES+64Bit","cipherlist_128Bit","cipherlist_3DES","cipher_order","secure_renego","secure_client_renego","fallback_SCSV","heartbleed","ticketbleed","CCS","ROBOT","POODLE_SSL","CRIME_TLS","BREACH","SWEET32","FREAK","DROWN","LOGJAM","BAR_MITZVAH","LUCKY13","RC4","BEAST","BEAST_CBC_TLS1"]
    list_of_all_ports = apiclient.findInDb(apiclient.getCurrentPentest(), "ports", { "_id": { "$in": port_ids } }, multi=True)
    list_of_all_lines = []
    list_of_defects = []
    dict_line = {}
    for port in list_of_all_ports:
        list_of_defects = port.get("infos", {}).get("TestSSL", [])
        if not list_of_defects:
            continue
        
        # Add missing defects as "OK" status
        for defect_id in expectedVulns:
            if not any(defect.get("defect") == defect_id for defect in list_of_defects):
                list_of_defects.append({"defect": defect_id, "criticity": "OK", "details": ""})
        for vuln in list_of_defects:
            dict_line = {"ip": port.get("ip",""), "port": port.get("port",""), "domain": port.get("infos", {}).get("FQDN",""), "defect": vuln.get("defect",""), "criticity": vuln.get("criticity",""), "details": vuln.get("details","")}
            list_of_all_lines.append(dict_line)
    vulnDataList = []
    completeData = []
    newTLSDefect = set()
    notAfter = None
    notBefore = None
    lifespan = -1
    keySizeInfo = None
    TLSv1_0_or_1_1 = False
    already_added = False
    
    for vuln in list_of_all_lines:
        # Check if SSLv2 is enabled
        vuln["defect"] = vuln["defect"].strip()
        if vuln["defect"] == "SSLv2":
            if vuln["criticity"] not in ["OK", "INFO"]:
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "SSLv2 Enabled", "criticity": vuln["criticity"]})
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "SSLv2", "vulnerable": "yes", "state": "Enabled"})
                # newTLSDefect.add(json.dumps(["SSLv2 activé", "7"]))
                continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "SSLv2", "vulnerable": "no", "state": "Disabled"})
        
        # Check if SSLv3 is enabled
        elif vuln["defect"] == "SSLv3":
            if vuln["criticity"] not in ["OK", "INFO"]:
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "SSLv3 Enabled", "criticity": vuln["criticity"]})
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "SSLv3", "vulnerable": "yes", "state": "Enabled"})
                newTLSDefect.add(json.dumps(["SSLv3 activé", "7"]))
                continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "SSLv3", "vulnerable": "no", "state": "Disabled"})
        
        # Check if TLSv1.0 is enabled
        elif vuln["defect"] == "TLS1":
            if vuln["criticity"] not in ["OK", "INFO"]:
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "TLSv1.0 Enabled", "criticity": vuln["criticity"]})
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "TLSv1.0", "vulnerable": "yes", "state": "Enabled"})
                TLSv1_0_or_1_1 = True
                continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "TLSv1.0", "vulnerable": "no", "state": "Disabled"})
        
        # Check if TLSv1.1 is enabled
        elif vuln["defect"] == "TLS1_1":
            if vuln["criticity"] not in ["OK", "INFO"]:
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "TLSv1.1 Enabled", "criticity": vuln["criticity"]})
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "TLSv1.1", "vulnerable": "yes", "state": "Enabled"})
                TLSv1_0_or_1_1 = True
                continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "TLSv1.1", "vulnerable": "no", "state": "Disabled"})
        
        # Check if one of the TLSv1.0 or TLSv1.1 is enabled
        elif TLSv1_0_or_1_1 == True and already_added == False:
            newTLSDefect.add(json.dumps(["TLSv1.0 et TLSv1.1 activés", "21,66"]))
            already_added = True
        
        # Check if TLSv1.2 is disabled
        if vuln["defect"] == "TLS1_2":
            if vuln["criticity"] not in ["OK", "INFO"]:
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "TLSv1.2 Disabled", "criticity": vuln["criticity"]})
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "TLSv1.2", "vulnerable": "yes", "state": "Disabled"})
                newTLSDefect.add(json.dumps(["TLSv1.2 et/ou TLSv1.3 non-offert", "66"]))
                continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "TLSv1.2", "vulnerable": "no", "state": "Enabled"})
        
        # Check if TLSv1.3 is disabled
        elif vuln["defect"] == "TLS1_3":
            if (vuln["criticity"] not in ["OK", "INFO"]) or ("not offered" in vuln["details"].lower()):
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "TLSv1.3 Disabled", "criticity": vuln["criticity"]})
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "TLSv1.3", "vulnerable": "yes", "state": "Disabled"})
                newTLSDefect.add(json.dumps(["TLSv1.2 et/ou TLSv1.3 non-offert", "66"]))
                continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "TLSv1.3", "vulnerable": "no", "state": "Enabled"})
        
        # Check if Perfect Forward Secrecy is disabled
        elif vuln["defect"] == "FS":
            if vuln["criticity"] not in ["OK", "INFO"]:
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "Perfect Forward Secrecy Disabled", "criticity": vuln["criticity"]})
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "PFS", "vulnerable": "yes", "state": "Disabled"})
                newTLSDefect.add(json.dumps(["Pas de Perfect Forward Secrecy", "21"]))
                continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "PFS", "vulnerable": "no", "state": "Enabled"})
        
        # Check if NULL/eNULL Cipher is enabled
        elif vuln["defect"] == "cipherlist_NULL":
            if vuln["criticity"] not in ["OK", "INFO"]:
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "NULL/eNULL Ciphersuites Enabled", "criticity": vuln["criticity"]})
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "NULL/eNULL Ciphersuites", "vulnerable": "yes", "state": "Enabled"})
                newTLSDefect.add(json.dumps(["Suites de chiffrement NULL/eNULL", "21"]))
                continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "NULL/eNULL Ciphersuites", "vulnerable": "no", "state": "Disabled"})
        
        # Check if NULL authentication is enabled
        elif vuln["defect"] == "cipherlist_aNULL":
            if vuln["criticity"] not in ["OK", "INFO"]:
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "NULL Authentication Ciphersuites Enabled", "criticity": vuln["criticity"]})
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "aNULL Ciphersuites", "vulnerable": "yes", "state": "Enabled"})
                newTLSDefect.add(json.dumps(["Authentification NULL", "21"]))
                continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "aNULL Ciphersuites", "vulnerable": "no", "state": "Disabled"})
        
        # Check if Export Cipher is enabled
        elif vuln["defect"] == "cipherlist_EXPORT":
            if vuln["criticity"] not in ["OK", "INFO"]:
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "Export Ciphersuites Enabled", "criticity": vuln["criticity"]})
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "EXPORT Ciphersuites", "vulnerable": "yes", "state": "Enabled"})
                newTLSDefect.add(json.dumps(["Suites de chiffrement « Export »", "21"]))
                continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "EXPORT Ciphersuites", "vulnerable": "no", "state": "Disabled"})
        
        # Check if DES Ciphersuites have a key length < 64 bits
        elif vuln["defect"] == "cipherlist_DES+64Bit":
            if vuln["criticity"] not in ["OK", "INFO"]:
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "DES Ciphersuites with keysize < 64 Bits", "criticity": vuln["criticity"]})
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "DES Ciphersuites with keysize < 64 Bits", "vulnerable": "yes", "state": "True"})
                # newTLSDefect.add(json.dumps(["Suites de chiffrement DES de clé < 64 bits", "21"]))
                continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "DES Ciphersuites with keysize < 64 Bits", "vulnerable": "no", "state": "False"})
        
        # Check if Ciphersuites have key length < 128 bits
        elif vuln["defect"] == "cipherlist_128Bit":
            if vuln["criticity"] not in ["OK", "INFO"]:
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "Ciphersuites with keysize < 128 Bits", "criticity": vuln["criticity"]})
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "Ciphersuites with keysize < 128 Bits", "vulnerable": "yes", "state": "True"})
                newTLSDefect.add(json.dumps(["Suites de chiffrement de clé < 128 bits", "21"]))
                continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "Ciphersuites with keysize < 128 Bits", "vulnerable": "no", "state": "False"})
        
        # Check if TripleDES Ciphersuites are enabled
        elif vuln["defect"] == "cipherlist_3DES":
            if vuln["criticity"] not in ["OK", "INFO"]:
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "TripleDES Ciphersuites Enabled", "criticity": vuln["criticity"]})
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "TripleDES Ciphersuites", "vulnerable": "yes", "state": "Enabled"})
                newTLSDefect.add(json.dumps(["Suites de chiffrement utilisant TripleDES", "21"]))
                continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "TripleDES Ciphersuites", "vulnerable": "no", "state": "Disabled"})
        
        # Check if AEAD is not available
        elif " " in vuln["defect"]:
            if vuln["criticity"] not in ["OK", "INFO"]:
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "AEAD Encryption not available", "criticity": vuln["criticity"]})
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "AEAD Encryption", "vulnerable": "yes", "state": "Disabled"})
                newTLSDefect.add(json.dumps(["Suites de chiffrement AEAD non disponibles", "21"]))
                continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "AEAD Encryption", "vulnerable": "no", "state": "Enabled"})
        
        # Check if Cipher ordering is disabled
        elif vuln["defect"] == "cipher_order":
            if vuln["criticity"] not in ["OK", "INFO"]:
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "Cipher Ordering Disabled", "criticity": vuln["criticity"]})
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "Cipher Ordering", "vulnerable": "yes", "state": "Disabled"})
                newTLSDefect.add(json.dumps(["Pas d'ordre dans les suites de chiffrement", "20"]))
                continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "Cipher Ordering", "vulnerable": "no", "state": "Enabled"})
        
        # Check if Secure renegotiation is disabled
        elif vuln["defect"] == "secure_renego":
            if vuln["criticity"] not in ["OK", "INFO"]:
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "Secure Renegociation Disabled", "criticity": vuln["criticity"]})
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "Secure Renegotiation", "vulnerable": "yes", "state": "Disabled"})
                newTLSDefect.add(json.dumps(["Renégociation sécurisée vulnérable (CVE-2009-3555)", "21"]))
                continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "Secure Renegotiation", "vulnerable": "no", "state": "Enabled"})
        
        # Check if Client-Initiated Renegotiation is enabled
        elif vuln["defect"] == "secure_client_renego":
            if vuln["criticity"] not in ["OK", "INFO"]:
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "Client-Initiated Renegociation Enabled", "criticity": vuln["criticity"]})
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "Client-Initiated Renegotiation", "vulnerable": "yes", "state": "Enabled"})
                newTLSDefect.add(json.dumps(["Renégociation sécurisée initiée par le client", "23"]))
                continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "Client-Initiated Renegotiation", "vulnerable": "no", "state": "Disabled"})
        
        # Check if TLS_FALLBACK_SCSV is disabled
        elif vuln["defect"] == "fallback_SCSV":
            if vuln["criticity"] not in ["OK", "INFO"]:
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "TLS_FALLBACK_SCSV Disabled", "criticity": vuln["criticity"]})
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "TLS_FALLBACK_SCSV", "vulnerable": "yes", "state": "Disabled"})
                newTLSDefect.add(json.dumps(["Absence de support de TLS_FALLBACK_SCSV", "21"]))
                continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "TLS_FALLBACK_SCSV", "vulnerable": "no", "state": "Enabled"})
        
        # CERTIFICATES
        
        # Get certificate common name
        elif vuln["defect"] in ["cert_commonName", "cert_commonName <hostCert#1>"]:
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "Common Name", "vulnerable": "no", "state": vuln["details"]})
        
        # Get certificate alternative names
        elif vuln["defect"] in ["cert_subjectAltName", "cert_subjecAltName <hostCert#1>"]:
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "Alternative Name(s)", "vulnerable": "no", "state": vuln["details"]})
        
        # Get certificate Validity left
        elif vuln["defect"] in ["cert_expirationStatus", "cert_expirationStatus <hostCert#1>"]:
            if vuln["criticity"] not in ["OK", "INFO"]:
                if vuln["details"] == "expired":
                    newTLSDefect.add(json.dumps(["Certificat expiré", "6"]))
                    vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "Certificate " + vuln["details"], "criticity": vuln["criticity"]})
                    completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "Validity Left", "vulnerable": "yes", "state": vuln["details"]})
                else:
                    vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "Certificate " + vuln["details"], "criticity": vuln["criticity"]})
                    completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "Validity Left", "vulnerable": "yes", "state": vuln["details"]})
            else:
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "Validity Left", "vulnerable": "no", "state": vuln["details"]})
        
        # Get notAfter date
        elif vuln["defect"] in ["cert_notAfter", "cert_notAfter <hostCert#1>"]:
            notAfter = datetime.strptime(vuln["details"], "%Y-%m-%d %H:%M") if vuln["details"] else None
        
        # Get notBefore date
        elif vuln["defect"] in ["cert_notBefore", "cert_notBefore <hostCert#1>"]:
            notBefore = datetime.strptime(vuln["details"], "%Y-%m-%d %H:%M") if vuln["details"] else None
        
        # Calculate certificate lifespan
        elif notAfter is not None and notBefore is not None and lifespan == -1:
            lifespan = abs((notAfter - notBefore).days)
            if lifespan > 398:
                newTLSDefect.add(json.dumps(["Durée de validité du certificat trop longue", "6"]))
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "Total Lifespan", "vulnerable": "yes", "state": str(lifespan) + " days"})
                continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "Total Lifespan", "vulnerable": "no", "state": str(lifespan) + " days"})
            lifespan = -1  # Reset lifespan for next certificate
            notAfter = None  # Reset notAfter for next certificate
            notBefore = None  # Reset notBefore for next certificate
        
        # Get certificate signature algorithm
        elif vuln["defect"] in ["cert_signatureAlgorithm", "cert_signatureAlgorithm <hostCert#1>"]:
            if vuln["criticity"] not in ["OK", "INFO"]:
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "Certificate " + vuln["details"], "criticity": vuln["criticity"]})
                if "SHA1" in vuln["details"]:
                    newTLSDefect.add(json.dumps(["Certificat signé en SHA-1", "6"]))
                    completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "Signature Algorithm", "vulnerable": "yes", "state": vuln["details"]})
                    continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "Signature Algorithm", "vulnerable": "no", "state": vuln["details"]})
        
        # Get Certificate key size
        elif vuln["defect"] in ["cert_keySize", "cert_keySize <hostCert#1>"]:
            if vuln["criticity"] not in ["OK", "INFO"]:
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "Certificate " + vuln["details"], "criticity": vuln["criticity"]})
                keySizeInfo = vuln["details"].split(" ")
                if (keySizeInfo[0] == "RSA" and int(keySizeInfo[1]) < 2048) or (keySizeInfo[0] == "EC" and int(keySizeInfo[1]) < 256):
                    newTLSDefect.add(json.dumps(["Taille de clé trop faible", "6"]))
                    completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "Key Size", "vulnerable": "yes", "state": vuln["details"]})
                    continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "Key Size", "vulnerable": "no", "state": vuln["details"]})
        
        # Get certificate issuer
        elif vuln["defect"] in ["cert_caIssuers", "cert_caIssuers <hostCert#1>"]:
            # Check if the issuer is DigiCert for SNCF only
            if is_sncf and "digicert" not in vuln["details"].lower():
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "Certificate issuer is not DigiCert (SNCF only)", "criticity": "INFO"})
                newTLSDefect.add(json.dumps(["Certificat émis par une autorité de certification non approuvée [SNCF uniquement, ne pas mentionner sinon]", "6"]))
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "Issuer", "vulnerable": "yes", "state": vuln["details"]})
                continue
            # Check if the issuer is emmited by a private CA or self-signed (not for SNCF)
            if ("selfsigned" in vuln["details"].lower() or "private" in vuln["details"].lower()) and not is_sncf:
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "Certificate issuer is self-signed or private", "criticity": "INFO"})
                newTLSDefect.add(json.dumps(["Certificat émis par une autorité de certification privée ou auto-signé", "6"]))
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "Issuer", "vulnerable": "yes", "state": vuln["details"]})
                continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "Issuer", "vulnerable": "no", "state": vuln["details"]})
        
        # Revocation list enabled ?
        elif vuln["defect"] in ["cert_crlDistributionPoints", "cert_crlDistributionPoints <hostCert#1>"]:
            if vuln["criticity"] not in ["OK", "INFO"]:
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "Certificate Revocation List Disabled", "criticity": vuln["criticity"]})
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "Revocation List", "vulnerable": "yes", "state": "Disabled"})
                newTLSDefect.add(json.dumps(["Absence de liste de révocation", "6"]))
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "Revocation List", "vulnerable": "no", "state": "Enabled"})
        
        # CA authorisation
        elif vuln["defect"] in ["DNS_CAArecord", "DNS_CAArecord <hostCert#1>"]:
            if vuln["criticity"] not in ["OK", "INFO"]:
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "No CA Authorisation", "criticity": vuln["criticity"]})
                newTLSDefect.add(json.dumps(["Pas d'autorisation d'Autorités de Certification (CAA)", "141"]))
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "CA Authorisation", "vulnerable": "yes", "state": "No CAA record found"})
                continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "CA Authorisation", "vulnerable": "no", "state": vuln["details"]})
        
        # Transparency
        elif vuln["defect"] in ["certificate_transparency", "certificate_transparency <hostCert#1>"]:
            if vuln["criticity"] not in ["OK", "INFO"]:
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "Certificate Transparancy Disabled", "criticity": vuln["criticity"]})
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "Transparency", "vulnerable": "yes", "state": "Disabled"})
                newTLSDefect.add(json.dumps(["Pas de Certificate Transparency", "6"]))
                continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "Transparency", "vulnerable": "no", "state": "Enabled"})
        
        # Chain of trust (SNCF is not concerned by this defect)
        elif vuln["defect"] in ["cert_chain_of_trust", "cert_chain_of_trust <hostCert#1>"]:
            if vuln["criticity"] not in ["OK", "INFO"]:
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "Chain of Trust Incomplete", "criticity": vuln["criticity"]})
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "Chain of Trust", "vulnerable": "yes", "state": "Incomplete"})
                if not is_sncf:
                    newTLSDefect.add(json.dumps(["Chaîne de confiance incomplète", "6"]))
                continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "Chain of Trust", "vulnerable": "no", "state": "Verified"})
        
        # VULNERABILITIES
        
        # Heartbleed
        elif vuln["defect"] == "heartbleed":
            if vuln["criticity"] not in ["OK", "INFO"]:
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "Heartbleed", "criticity": vuln["criticity"]})
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "Heartbleed", "vulnerable": "yes", "state": "Vulnerable"})
                # newTLSDefect.add(json.dumps(["Vulnérabilité Heartbleed", "21"]))
                continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "Heartbleed", "vulnerable": "no", "state": "Not Vulnerable"})
        
        # Ticketbleed
        elif vuln["defect"] == "ticketbleed":
            if vuln["criticity"] not in ["OK", "INFO"]:
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "Ticketbleed", "criticity": vuln["criticity"]})
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "Ticketbleed", "vulnerable": "yes", "state": "Vulnerable"})
                # newTLSDefect.add(json.dumps(["Vulnérabilité Ticketbleed", "21"]))
                continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "Ticketbleed", "vulnerable": "no", "state": "Not Vulnerable"})
        
        # CCS
        elif vuln["defect"] == "CCS":
            if vuln["criticity"] not in ["OK", "INFO"]:
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "CCS Injection", "criticity": vuln["criticity"]})
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "CCS Injection", "vulnerable": "yes", "state": "Vulnerable"})
                newTLSDefect.add(json.dumps(["Vulnérabilité à la faille CCS (CVE-2014-0224)", "21"]))
                continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "CCS Injection", "vulnerable": "no", "state": "Not Vulnerable"})
        
        # ROBOT
        elif vuln["defect"] == "ROBOT":
            if vuln["criticity"] not in ["OK", "INFO"]:
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "ROBOT", "criticity": vuln["criticity"]})
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "ROBOT", "vulnerable": "yes", "state": "Vulnerable"})
                newTLSDefect.add(json.dumps(["Vulnérabilité à l'attaque ROBOT (CVE-2017-13099)", "21"]))
                continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "ROBOT", "vulnerable": "no", "state": "Not Vulnerable"})
        
        # POODLE
        elif vuln["defect"] == "POODLE_SSL":
            if vuln["criticity"] not in ["OK", "INFO"]:
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "POODLE", "criticity": vuln["criticity"]})
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "POODLE", "vulnerable": "yes", "state": "Vulnerable"})
                newTLSDefect.add(json.dumps(["Vulnérabilité à la faille POODLE (CVE-2014-3566)", "7"]))
                continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "POODLE", "vulnerable": "no", "state": "Not Vulnerable"})
        
        # CRIME
        elif vuln["defect"] == "CRIME_TLS":
            if vuln["criticity"] not in ["OK", "INFO"]:
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "CRIME", "criticity": vuln["criticity"]})
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "CRIME", "vulnerable": "yes", "state": "Vulnerable"})
                newTLSDefect.add(json.dumps(["Vulnérabilité à la faille CRIME (CVE-2012-4929)", "26"]))
                continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "CRIME", "vulnerable": "no", "state": "Not Vulnerable"})
        
        # BREACH
        elif vuln["defect"] == "BREACH":
            if vuln["criticity"] not in ["OK", "INFO"]:
                if "test failed" not in vuln["details"].lower():
                    vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "BREACH", "criticity": vuln["criticity"]})
                    completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "BREACH", "vulnerable": "yes", "state": "Vulnerable"})
                    newTLSDefect.add(json.dumps(["Vulnérabilité à BREACH", "25"]))
                    continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "BREACH", "vulnerable": "no", "state": "Not Vulnerable"})
        
        # SWEET32
        elif vuln["defect"] == "SWEET32":
            if vuln["criticity"] not in ["OK", "INFO"]:
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "SWEET32", "criticity": vuln["criticity"]})
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "SWEET32", "vulnerable": "yes", "state": "Vulnerable"})
                newTLSDefect.add(json.dumps(["Vulnérabilité à la faille SWEET32 (CVE-2016-2183)", "21"]))
                continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "SWEET32", "vulnerable": "no", "state": "Not Vulnerable"})
        
        # FREAK
        elif vuln["defect"] == "FREAK":
            if vuln["criticity"] not in ["OK", "INFO"]:
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "FREAK", "criticity": vuln["criticity"]})
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "FREAK", "vulnerable": "yes", "state": "Vulnerable"})
                newTLSDefect.add(json.dumps(["Vulnérabilité à la faille FREAK (CVE-2015-0204)", "21"]))
                continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "FREAK", "vulnerable": "no", "state": "Not Vulnerable"})
        
        # DROWN
        elif vuln["defect"] == "DROWN":
            if vuln["criticity"] not in ["OK", "INFO"]:
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "DROWN", "criticity": vuln["criticity"]})
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "DROWN", "vulnerable": "yes", "state": "Vulnerable"})
                newTLSDefect.add(json.dumps(["Vulnérabilité à la faille DROWN (CVE-2016-0800, CVE-2016-0703)", "7"]))
                continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "DROWN", "vulnerable": "no", "state": "Not Vulnerable"})
        
        # LOGJAM
        elif vuln["defect"] == "LOGJAM":
            if vuln["criticity"] not in ["OK", "INFO"]:
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "LOGJAM", "criticity": vuln["criticity"]})
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "LOGJAM", "vulnerable": "yes", "state": "Vulnerable"})
                newTLSDefect.add(json.dumps(["Vulnérabilité à la faille LOGJAM (CVE-2015-4000)", "21"]))
                continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "LOGJAM", "vulnerable": "no", "state": "Not Vulnerable"})
        
        # BAR-MITZVAH
        elif vuln["defect"] == "BAR_MITZVAH":
            if vuln["criticity"] not in ["OK", "INFO"]:
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "BAR-MITZVAH", "criticity": vuln["criticity"]})
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "BAR-MITZVAH", "vulnerable": "yes", "state": "Vulnerable"})
                newTLSDefect.add(json.dumps(["Vulnérabilité à la faille BAR-MITZVAH (CVE-2015-2808)", "21"]))
                continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "BAR-MITZVAH", "vulnerable": "no", "state": "Not Vulnerable"})
        
        # LUCKY13
        elif vuln["defect"] == "LUCKY13":
            if vuln["criticity"] not in ["OK", "INFO"]:
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "LUCKY13", "criticity": vuln["criticity"]})
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "LUCKY13", "vulnerable": "yes", "state": "Vulnerable"})
                newTLSDefect.add(json.dumps(["Vulnérabilité à la faille LUCKY13 (CVE-2013-0169)", "21"]))
                continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "LUCKY13", "vulnerable": "no", "state": "Not Vulnerable"})
        
        # RC4
        elif vuln["defect"] == "RC4":
            if vuln["criticity"] not in ["OK", "INFO"]:
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "RC4", "criticity": vuln["criticity"]})
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "RC4", "vulnerable": "yes", "state": "Vulnerable"})
                # newTLSDefect.add(json.dumps(["Vulnérabilité à la faille RC4", "21"]))
                continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "RC4", "vulnerable": "no", "state": "Not Vulnerable"})
        
        # BEAST
        elif vuln["defect"] == "BEAST":
            if vuln["criticity"] not in ["OK", "INFO"]:
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "BEAST", "criticity": vuln["criticity"]})
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "BEAST", "vulnerable": "yes", "state": "Vulnerable"})
                newTLSDefect.add(json.dumps(["Vulnérabilité à la faille BEAST (CVE-2011-3389)", "21"]))
                continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "BEAST", "vulnerable": "no", "state": "Not Vulnerable"})
        
        # BEAST CBC TLS1.0
        elif vuln["defect"] == "BEAST_CBC_TLS1":
            if vuln["criticity"] not in ["OK", "INFO"]:
                vulnDataList.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "defect": "BEAST CBC TLS1.0", "criticity": vuln["criticity"]})
                completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "BEAST CBC TLS1.0", "vulnerable": "yes", "state": "Vulnerable"})
                newTLSDefect.add(json.dumps(["Vulnérabilité à la faille BEAST (CVE-2011-3389)", "21"]))
                continue
            completeData.append({"ip": vuln["ip"], "port": vuln["port"], "domain": vuln["domain"], "about": "BEAST CBC TLS1.0", "vulnerable": "no", "state": "Not Vulnerable"})
    
        # Convert set to list and parse JSON strings back to arrays
    newTLSDefect = list(newTLSDefect)
    newTLSDefect = [json.loads(item) for item in newTLSDefect]
    
    return {"vulnTable": vulnDataList, "allDataForExcel": completeData, "newTLSDefectForReport": newTLSDefect}

def updateTLSDefect(defectTLS, new_defect, defect_to_keep, fixes_to_keep):
    defectTLS_description = defectTLS[0]["description"]
    if new_defect.get("description", "") == "":
        new_defect["description"] = defectTLS_description.split("\r\n\r\n")[0] + "\r\n\r\n"
    lignes = defectTLS_description.split("\r\n\r\n")
    title = ""
    for ligne in lignes:
        if ligne.startswith("#"):
            title = ligne.strip("#").strip()
            if title == defect_to_keep:
                new_defect["description"] += ligne + "\r\n\r\n"
                continue
        if title == defect_to_keep:
            new_defect["description"] += ligne + "\r\n\r\n"
            continue
    fixes_to_keep = fixes_to_keep.split(",")
    defectTLS_fixes = defectTLS[0]["fixes"]
    for fix in defectTLS_fixes:
        if str(fix["id"]) in fixes_to_keep and not fix in new_defect["fixes"]:
            new_defect["fixes"].append(fix)
    return new_defect
    
def generateDefect(apiclient, newTLSDefect):
    # Function to generate a defect in the report
    # This function should be implemented to handle the defect creation logic
    print("Generating defect with data:", newTLSDefect)
    defectTLS = apiclient.searchDefect("Défauts d'implémentation du TLS")[0]
    new_TLS_defect = {}
    new_TLS_defect["description"] = ""
    new_TLS_defect["fixes"] = []
    for d in newTLSDefect:
        new_TLS_defect = updateTLSDefect(defectTLS, new_TLS_defect, d[0], d[1])
    defectTLS[0]["description"] = new_TLS_defect["description"]
    defectTLS[0]["fixes"] = new_TLS_defect["fixes"]
    new_defect = defectTLS[0]
    all_defects = apiclient.getDefectTable()
    for d in all_defects:
        if d["title"] == new_defect["title"]:
            print("Defect already exists, updating it.")
            if "_id" in new_defect:
                del new_defect["_id"]
            apiclient.updateInDb(apiclient.getCurrentPentest(), "defects", {"_id":ObjectId(d["_id"])}, {"$set": new_defect})
            return {"status": "success", "data": new_defect}
    apiclient.insertInDb(apiclient.getCurrentPentest(), "defects", new_defect)
    return {"status": "success", "data": new_defect}

def generateExcel(excelData):
    Ips = []
    columnHeaders = ["IP", "Port", "Domain"]
    table = [["IP", "Port", "Domain"]]
    colorTable = [["no", "no", "no"]]
    for data in excelData:
        try:
            row_index = Ips.index(data.get("ip","")+"/"+data.get("port",""))
        except ValueError:
            row_index = len(Ips)
            Ips.append(data.get("ip","") + "/"+data.get("port",""))
            new_row = [""]*len(columnHeaders)
            new_row[0] = data.get("ip","")
            new_row[1] = data.get("port","")
            new_row[2] = data.get("domain","") or "N/A"
            table.append(new_row)
            colorTable.append(["no", "no", "no"])
        if data.get("about","") not in columnHeaders:
            columnHeaders.append(data.get("about",""))
            table[0].append(data.get("about",""))
        try:
            column_index = columnHeaders.index(data.get("about",""))
        except ValueError:
            column_index = -1
        if column_index == len(table[row_index + 1]):
            table[row_index + 1].append("")
        
        table[row_index + 1][column_index] = data.get("state","")
        if column_index == len(colorTable[row_index + 1]):
            colorTable[row_index + 1].append("")
        colorTable[row_index + 1][column_index] = data.get("vulnerable","")

    
    wb = Workbook()
    ws = wb.active
    ws.title = "TestSSL Report"
    
    # Add data to worksheet
    for row_idx, row_data in enumerate(table):
        for col_idx, cell_data in enumerate(row_data):
            cell = ws.cell(row=row_idx + 1, column=col_idx + 1, value=cell_data)
    
    # Color all the headers
    header_fill = PatternFill(start_color="95cd41", end_color="95cd41", fill_type="solid")
    for i in range(len(columnHeaders)):
        cell = ws.cell(row=1, column=i + 1)
        cell.fill = header_fill
        cell.alignment = Alignment(horizontal="center", vertical="center")
        cell.font = Font(bold=True)
    
    # Specify column width and cell formatting
    for i, column_header in enumerate(columnHeaders):
        max_length = len(column_header)
        for j, row_data in enumerate(table[1:], 1):
            try:
                if len(str(row_data[i])) > max_length:
                    max_length = len(str(row_data[i]))
                
                cell = ws.cell(row=j + 1, column=i + 1)
                cell.alignment = Alignment(horizontal="center", vertical="center")
                # center text in cell
                if colorTable[j][i] == "yes":
                    # Set font color in red and bold
                    cell.font = Font(color="FF0000", bold=True)
                    cell.alignment = Alignment(horizontal="center", vertical="center")
            except (IndexError, TypeError) as error:
                print("Index/Type error while processing table cell:", error)
        
        size = max_length * 1.2
        # Set column width
        column_letter = get_column_letter(i + 1)
        ws.column_dimensions[column_letter].width = size
    
    # Specify row height
    ws.row_dimensions[1].height = 15  # set the first row height (header row)
    for i in range(2, len(table) + 1):
        ws.row_dimensions[i].height = 50
    
    # Save the workbook
    wb.save("TestSSL_Report.xlsx")
    return "TestSSL_Report.xlsx"

    # Code to generate Excel file goes here

def main(apiclient, *args, **kwargs):
    # Example usage of the parseTestSSL function
    results = testsslresults(apiclient)
    if results:
        list_of_defects = results.get("vulnTable", [])
        print("Défauts trouvés :")
        print("\n".join(set([d["defect"] for d in list_of_defects])))
        response = input("Créer le défaut dans le rapport ? (y/N): ")
        if response.lower() == "y":
            results = generateDefect(apiclient, results.get("newTLSDefectForReport", []))
            if results["status"] == "success":
                print("Défaut créé avec succès ")
            else:
                print("Erreur dans la création du défaut.")
        response = input("Générer un fichier Excel d'annexe ? (y/N): ")
        if response.lower() == "y":
            # Code to generate Excel report goes here
            file_path = generateExcel(results.get("allDataForExcel", []))
            res = apiclient.uploadFile(file_path)
            if res:
                print("Fichier Excel TestSSL_Report.xlsx généré et uploadé avec succès.")
            else:
                print("Erreur lors de l'upload du fichier Excel.", res)
        return True, "Fichier Excel TestSSL_Report.xlsx généré et uploadé avec succès."    
    return False, "No results found or an error occurred."
# apiclient = APIClient.getInstance()  # Assuming APIClient is defined elsewhere
# apiclient.tryConnection()
# res = apiclient.tryAuth()
# if not res:
#     consoleConnect()
# if apiclient.isConnected() is False or apiclient.getCurrentPentest() == "":
#     print("You must be connected to a pentest to run this script.")

# main(apiclient)



from pollenisatorgui.core.components.apiclient import APIClient

def find_asreproastable_users(apiclient):
    users_data = apiclient.findInDb(apiclient.getCurrentPentest(), "users", { "infos.asreproastable":True}, multi=True)
    if not users_data:
        print("No asreproastable users found.")
        return []
    return list(users_data)

    
def generateDefect(apiclient, users):
    defectASREP = apiclient.searchDefect("ASREPRoast")[0]
    description = """L'attaque ASREPRoast tire parti des **comptes qui sont configurés pour ne pas nécessiter de pré-authentication Kerberos**[^https://support.microsoft.com/en-us/help/305144/how-to-use-the-useraccountcontrol-flags-to-manipulate-user-account-pro] afin de récupérer un ticket TGT sans inclure de données de pré-authentification dans la première phase de négociation, appelée AS-REQ. La réponse AS-REP retournée par le centre de distribution de clés, le KDC, inclut une clé de session chiffrée avec celle dérivée du mot de passe de l’utilisateur. En d'autres termes, **le ticket ainsi obtenu contient l’empreinte du mot de passe du compte**. Il est alors possible de tenter de **casser cette empreinte pour récupérer le mot de passe en clair**, à l’aide d’une machine dédiée à cet effet. Moins le mot de passe est solide, plus son mot de passe sera récupéré rapidement.

Ici, les comptes concernés, c'est à dire disposant d'un SPN et de l'attribut **DONT_REQ_PREAUTH**, sont les suivants : """
    description += "\n\n"
    description += "| Nom d'utilisateur | Domaine |\n"
    description += "|------------------|---------|\n"
    for u in users:
        description += f"| {u['username']} | {u['domain']} |\n"
    description += "\n"
    defectASREP[0]["description"] = description
    new_defect = defectASREP[0]
    all_defects = apiclient.getDefectTable()
    for d in all_defects:
        if d["defect_id"] == new_defect["defect_id"]:
            print("Defect already exists, updating it.")
            if "_id" in new_defect:
                del new_defect["_id"]
            apiclient.updateInDb(apiclient.getCurrentPentest(), "defects", {"_id":ObjectId(d["_id"])}, {"$set": new_defect})
            return {"status": "success", "data": new_defect}
    apiclient.insertInDb(apiclient.getCurrentPentest(), "defects", new_defect)
    return {"status": "success", "data": new_defect}


def main(apiclient, *args, **kwargs):
    # Example usage of the parseTestSSL function
    users = find_asreproastable_users(apiclient)
    if not users:
        print("No asreproastable users found.")
        return True
    if len(users) > 0:
        print("Asreproastable users found:", len(users))
        response = input("Créer le défaut dans le rapport ? (y/N): ")
        if response.lower() == "N":
            return True
        results = generateDefect(apiclient, users)
        if results["status"] == "success":
            print("Défaut créé avec succès.")
            return True
        else:
            print("Erreur dans la création du défaut.")
            return False, "Erreur dans la création du défaut."
        
    return False, "No results found or an error occurred."



import threading
import time
import socketio
from pollenisatorgui.core.components.apiclient import APIClient
import pollenisatorgui.core.components.utils as utils
from pollenisatorgui.core.components.logger_config import logger
from pollenisatorgui.pollenisator import consoleConnect
from pollenisatorgui.core.components.scanworker import ScanWorker
import pty
import os
import subprocess
import select
import termios
import struct
import fcntl
import shlex

def set_winsize(fd, row, col, xpix=0, ypix=0):
    logger.debug("setting window size with termios")
    winsize = struct.pack("HHHH", row, col, xpix, ypix)
    fcntl.ioctl(fd, termios.TIOCSWINSZ, winsize)

class TerminalWorker(ScanWorker):
    
    def __init__(self, local_settings):
        super().__init__(local_settings)
        self.name = ""
        self.pid = None
        self.fd = None
        self.connected = False
        self.sessions = {}
        terminal, _ = utils.getPreferedShell()
        self.cmd = [terminal]


         
    def read_and_forward_pty_output(self, session_id):
        max_read_bytes = 1024 * 20
        fd = self.sessions[session_id]["fd"]
        while True:
            time.sleep(0.01)
            if fd:
                timeout_sec = 0
                (data_ready, _, _) = select.select([fd], [], [], timeout_sec)
                if data_ready:
                    output = os.read(fd, max_read_bytes).decode(
                        errors="ignore"
                    )
                    self.sio.emit("proxy-term", {"action":"pty-output", "id":session_id, "output": output})

    def startTerminalSession(self, data):
        session_id = data.get("id", None)
        target_check_iid = data.get("target_check_iid", None)
        if session_id is None:
            return
        if session_id in self.sessions:
            return
        self.sessions[session_id] = {
            "fd": None,
            "pid": None,
            "cmd": self.cmd,
            "timer": None,
            "connected": False,
            "target_check_iid": target_check_iid,
            "target_tools_iids": data.get("target_tools_iids", None)
        }
        (child_pid, fd) = pty.fork()
        if child_pid != 0:
            self.sessions[session_id]["fd"] = fd
            self.sessions[session_id]["pid"] = child_pid
            set_winsize(fd, data.get("dims", {}).get("rows", 50), data.get("dims", {}).get("cols", 50))
            cmd = " ".join(shlex.quote(c) for c in self.cmd)
            # logging/print statements must go after this because... I have no idea why
            # but if they come before the background task never starts
            t = threading.Thread(target=self.read_and_forward_pty_output, args=(session_id,))
            t.start()
            logger.info("child pid is " + str(child_pid))
            logger.info(
                f"starting background task with command `{cmd}` to continously read "
                "and forward pty output to client"
            )
            logger.info("task started")
        else:
            environ = os.environ.copy()
            target_check_iid = self.sessions[session_id].get("target_check_iid", None)
            target_tools_iids = self.sessions[session_id].get("target_tools_iids", None)
            target = ""
            if target_check_iid:
                if isinstance(target_check_iid, list):
                    target = ",".join(target_check_iid)
                else:
                    target = str(target_check_iid)
            if target_tools_iids:
                target += "|"
                if isinstance(target_tools_iids, list):
                    target += ",".join(target_tools_iids)
                else:
                    target += str(target_tools_iids)
            environ["POLLENISATOR_DEFAULT_TARGET"] = target
            logger.info("starting terminal session with command: %s" % self.cmd)
            subprocess.run(self.cmd,  shell=True, env=environ)
        
    def stopTerminalSession(self, data):
        session_id = data.get("id", None)
        if session_id is None:
            return
        if session_id not in self.sessions:
            return
        session = self.sessions[session_id]
        if session["fd"]:
            os.close(session["fd"])
        if session["pid"]:
            os.kill(session["pid"], 9)
        del self.sessions[session_id]

    def sendInput(self, data):
        session_id = data.get("id", None)
        if session_id is None:
            return
        if session_id not in self.sessions:
            return
        session = self.sessions[session_id]
        if session["fd"]:
            logger.debug("received input from browser: %s" % data["input"])
            os.write(session["fd"], data["input"].encode())

    def sendStop(self, session):
        if session["fd"]:
            print("sending Ctrl+c to session")
            os.write(session["fd"], b"\x03")
    
    def resize(self, data):
        session_id = data.get("id", None)
        if session_id is None:
            return
        if session_id not in self.sessions:
            return
        session = self.sessions[session_id]
        if session["fd"]:
            dims = data.get("dims", {})
            logger.debug(f"Resizing window to {dims['rows']}x{dims['cols']}")
            set_winsize(session["fd"], dims["rows"], dims["cols"])

    def connect(self, name, plugins, force_reconnect=False):
        apiclient = APIClient.getInstance()
        if force_reconnect:
            apiclient.disconnect()
        self.name = name
        apiclient.tryConnection()
        res = apiclient.tryAuth()
        if not res:
            consoleConnect()
        if apiclient.isConnected() is False or apiclient.getCurrentPentest() == "":
            return
        findPlugins = apiclient.getPlugins()
        if findPlugins is not None:
            for pluginFound in findPlugins:
                if pluginFound["plugin"] not in self.local_settings["my_commands"].keys():
                    # NOT CONFIGURED YET
                    if pluginFound["plugin"] == "auto-detect" or pluginFound["plugin"] == "Default":
                        # auto-detect is a special case, we don't want to configure it
                        continue
                    get_bin_path = utils.which_expand_aliases(pluginFound["default_bin_names"])
                    for value in get_bin_path.values():
                        if value is not None:
                            get_bin_path = value
                            break
                    if get_bin_path is not None:
                        # BUT TOOL IS INSTALLED
                        self.local_settings["my_commands"][pluginFound["plugin"]] = get_bin_path
                        utils.save_local_settings(self.local_settings)
                        plugins.append(pluginFound["plugin"])
                        print("[+] Found missing plugin : auto configuration "+pluginFound["plugin"]+" with path "+get_bin_path)
                    else:
                        print("[-] Missing plugin : "+pluginFound["plugin"]+" (not installed on this system?)")
        self.sio.connect(apiclient.api_url)
        self.sio.emit("registerAsTerminalWorker", {"token":apiclient.getToken(), "name":name, "supported_plugins":plugins, "pentest":apiclient.getCurrentPentest()})
        self.connected = False
        @self.sio.on("testTerminal")
        def test(data):
            print("Got terminal test "+str(data))

        @self.sio.on("consumer_connected")
        def consumer_connected(data):
            print("Got terminal consumer_connected "+str(data))
            self.connected = True

        @self.sio.on("consumer_disconnected")
        def consumer_disconnected(data):
            print("Got terminal consumer_disconnected "+str(data))
            self.connected = False

        @self.sio.on("proxy-term")
        def proxy_term(data):
            print("proxy-term "+str(data))
            if data.get("action", "") == "start-terminal-session":
                self.startTerminalSession(data)
            if data.get("action", "") == "stop-terminal-session":
                self.stopTerminalSession(data)
            if data.get("action", "") == "pty-input":
                self.sendInput(data)
            if data.get("action", "") == "pty-resize":
                self.resize(data)

        @self.sio.on("stop-terminal-command")
        def stop_terminal_command(data):
            print("Got stop-terminal-command "+str(data))
            for session_id, session_data in self.sessions.items():
                target = session_data.get("target_check_iid")
                print("Target is "+str(target))
                parts = target.split("|")
                if len(parts) >= 2:
                    tool_iid = parts[1]
                    print("If tool iid is "+str(tool_iid)+" == "+str(data.get("tool_iid")))
                    if tool_iid == data.get("tool_iid"):
                        print("SendingStop to session if fd")
                        self.sendStop(session_data)

        super().connect(name)


import re
import sys
import shlex
import multiprocessing
import time
from pollenisatorgui.core.components.logger_config import logger
from pollenisatorgui.core.components.utils import getDataFolder

def pollex():
    if len(sys.argv) <= 1:
        print("Usage : pollex [-v] [--checkinstance <checkinstance_id> | <Plugin name> <command options> | <command to execute>]")
        sys.exit(1)
    verbose = False
    if sys.argv[1] == "-v":
        verbose = True
    if "--checkinstance" in sys.argv:
        try:
            index_check = sys.argv.index("--checkinstance")
            script_checkinstance_id = sys.argv[index_check+1]
            pollscript_exec(script_checkinstance_id,  verbose)
            return
        except IndexError as e:
            print("ERROR : --checkinstance option must be followed by a checkinstance id")
            sys.exit(1)
    else:
        if sys.argv[1] == "-v":
            execCmd = shlex.join(sys.argv[2:])
        else:
            execCmd = shlex.join(sys.argv[1:])
    output = pollex_exec(execCmd, verbose)
    if output is not None:
        print("Result file : "+str(output))

def pollscript_exec(script_checkinstance_id, verbose=False):
    import os
    import tempfile
    import time
    import shutil
    from pollenisatorgui.core.components.apiclient import APIClient
    from pollenisatorgui.core.models.checkitem import CheckItem
    from pollenisatorgui.core.models.checkinstance import CheckInstance
    from bson import ObjectId
    from pollenisatorgui.pollenisator import consoleConnect, parseDefaultTarget
    import pollenisatorgui.core.components.utils as utils
    import importlib

    apiclient = APIClient.getInstance()
    apiclient.tryConnection()
    res = apiclient.tryAuth()
    if not res:
        # Check if we have a current pentest configured before asking for one
        current_pentest = apiclient.getCurrentPentest()
        ask_pentest = not current_pentest or current_pentest.strip() == ""
        consoleConnect(askPentest=ask_pentest)
    check_instance = CheckInstance.fetchObject({"_id":ObjectId(script_checkinstance_id)})
    if check_instance is None:
        print("ERROR : CheckInstance not found")
        return
    script_checkitem_id = check_instance.check_iid
    check_o = CheckItem.fetchObject({"_id":ObjectId(script_checkitem_id)})
    if check_o is None:
        print("ERROR : Check not found")
        return
    if check_o.check_type != "script":
        print("ERROR : Check is not a script check")
        return
    
    script_name = os.path.normpath(check_o.title).replace(" ", "_").replace("/", "_").replace("\\", "_")+".py"
    default_target = parseDefaultTarget(os.environ.get("POLLENISATOR_DEFAULT_TARGET", ""))
    results_folder = getDataFolder() ### HACK: tempfile.TemporaryDirectory() gets deleted early because a fork occurs in execute and atexit triggers.
    script_path = os.path.normpath(os.path.join(results_folder, script_name))
    with open(script_path, "w") as f:
        f.write(check_o.script)
    spec = importlib.util.spec_from_file_location("pollenisatorgui.scripts."+str(script_name), script_path)
    script_module = importlib.util.module_from_spec(spec)
    sys.modules["pollenisatorgui.scripts."+str(script_name)] = script_module
    spec.loader.exec_module(script_module)
    data = check_instance.getData()
    data["default_target"] = str(check_instance.getId())
    success, res = script_module.main(APIClient.getInstance(), None, **data)
    if success:
        print(f"Script {script_name} finished.\n{res}")
    else:
        print(f"Script {script_name} failed.\n{res}")

def pollex_exec(execCmd, verbose=False):
    """Send a command to execute for pollenisator-gui running instance
    """
    
    bin_name = shlex.split(execCmd)[0]
    if bin_name in ["echo", "print", "vim", "vi", "tmux", "nano", "code", "cd", "ls","pwd", "cat", "export"]:
        sys.exit(-1)
    import os
    from pollenisatorgui.core.components.apiclient import APIClient
    from pollenisatorgui.pollenisator import consoleConnect, parseDefaultTarget
    from datetime import datetime
    import pollenisatorgui.core.components.utils as utils

    cmdName = os.path.splitext(os.path.basename(bin_name))[0]
    apiclient = APIClient.getInstance()
    apiclient.tryConnection()
    res = apiclient.tryAuth()
    if not res:
        consoleConnect()
    if apiclient.isConnected() is False or apiclient.getCurrentPentest() == "":
        return
    res = apiclient.getDesiredOutputForPlugin(execCmd, "auto-detect")
    (success, data) = res
    if not success:
        print(data)
        return
    cmdName += "_" + datetime.now().strftime("%Y-%m-%d_%H-%M-%S")
    default_target = parseDefaultTarget(os.environ.get("POLLENISATOR_DEFAULT_TARGET", ""))
    tools_iids = default_target.get("tool_iid") 
    if tools_iids is not None:
        if not isinstance(tools_iids, list):
            tools_iids = [tools_iids]
        for tool_iid in tools_iids:
            apiclient.setToolStatus(tool_iid, ["running"])
    
    if not success:
        print("ERROR : "+data)
        return
    if not data:
        print("ERROR : An error as occured : "+str(data))
        return
    local_settings = utils.load_local_settings()
    my_commands = local_settings.get("my_commands", {})
    path_to_check = set()
    bin_path = my_commands.get(bin_name, None)
    if bin_path is not None:
        path_to_check.add(bin_path)
    path_to_check.add(bin_name)
    plugin_results = data["plugin_results"]
    for plugin, plugin_data in plugin_results.items():
        if os.path.splitext(plugin)[0] in execCmd:
            path_to_check = path_to_check.union(plugin_data.get("common_bin_names", []))
    bin_path_found, result_msg = utils.checkPaths(list(path_to_check))
    if not bin_path_found:
        print("ERROR : "+result_msg)
        return
    new_bin_path = result_msg
    comm = data["command_line_options"].replace(bin_name, new_bin_path, 1)
    
    if (verbose):
        print("INFO : Matching plugins are "+str(data["plugin_results"]))

    files_attached = re.findall(r'\|file\_[a-f0-9\-]+\|', comm)
    for f in files_attached:
        file_id = f.replace("|file_", "").replace("|", "")
        file_path = apiclient.downloadById("file", file_id, file_id, "/tmp/"+file_id)
        comm = comm.replace(f"|file_{file_id}|", file_path)
    
    result_dir = getDataFolder() 
    
    for plugin, plugin_data in plugin_results.items():
        ext = plugin_data.get("expected_extension", ".log.txt")
    
        outputFileDir= os.path.join(result_dir, os.path.splitext(plugin)[0])
        os.makedirs(outputFileDir, exist_ok=True)
        outputFilePath = os.path.join(outputFileDir, cmdName) + ext
        comm = comm.replace(f"|{plugin}.outputDir|", outputFilePath)
    if (verbose):
        print("Executing command : "+str(comm))
        print("output should be in "+str(outputFilePath))
    queue = multiprocessing.Queue()
    queueResponse = multiprocessing.Queue()
    #if comm.startswith("sudo "):
    #    returncode = utils.execute_no_fork(comm, None, True, queue, queueResponse, cwd=tmpdirname)
    #else:
    try:
        # getcwd is needed to have a valid cwd in case of relative paths in the command (filepath to a userlist for example)
        returncode = utils.execute(comm, None, queue, queueResponse, cwd=os.getcwd(), printStdout=True)
    except KeyboardInterrupt:
        logger.debug("pollex KeyboardInterrupt for comm "+str(comm))
    except Exception as e:
        logger.debug("pollex Exception for comm "+str(comm)+" "+str(e))
    queue.put("kill", block=False)
    if len(plugin_results) == 1 and "Default" in plugin_results:
        if (verbose):
            print("INFO : Only default plugin found")
        response = input("No plugin matched, do you want to use default plugin to log the command and stdout ? (Y/n) :")
        if str(response).strip().lower() == "n":
            return
    logger.debug("pollex detect plugins "+str(plugin_results))
    atLeastOne = False
    error = ""
    for plugin, plugin_data in plugin_results.items():
        ext = plugin_data.get("expected_extension", ".log.txt")
        outputFilePath = os.path.join(result_dir, os.path.splitext(plugin)[0], cmdName) + ext
        if not os.path.exists(outputFilePath):
            if os.path.exists(outputFilePath+ext):
                outputFilePath+=ext
            else:
                print(f"ERROR : Expected file was not generated {outputFilePath}")
                error = "ERROR : Expected file was not generated"
                continue
        print(f"INFO : Uploading results {outputFilePath} (async)")
        
        # Use async upload
        try:
            response = apiclient.importExistingResultFileAsync(outputFilePath, plugin, default_target, comm)
            task_id = response.get("task_id")
            print(f"INFO : Upload queued with task ID: {task_id}")
            
            # Poll for completion
            max_wait_time = 300  # 5 minutes timeout
            poll_interval = 2  # Poll every 2 seconds
            elapsed_time = 0
            
            while elapsed_time < max_wait_time:
                try:
                    result = apiclient.getImportTaskResult(task_id)
                    status = result.get("status")
                    
                    if status == "completed":
                        results = result.get("results", {})
                        print(f"INFO : Import completed successfully: {results}")
                        atLeastOne = True
                        break
                    elif status in ["queued", "processing"]:
                        print(f"INFO : Import {status}... (elapsed: {elapsed_time}s)")
                        time.sleep(poll_interval)
                        elapsed_time += poll_interval
                    else:
                        # Should not reach here as failed status raises an exception
                        error = result.get("error", "Unknown error")
                        print(f"ERROR : Import failed: {error}")
                        break
                        
                except Exception as e:
                    # Check if it's a failed task (400 error)
                    if hasattr(e, 'response') and e.response.status_code == 400:
                        task_error = e.ret_values[0] if e.ret_values else str(e)
                        if isinstance(task_error, dict):
                            error = task_error.get("error", str(task_error))
                        else:
                            error = str(task_error)
                        print(f"ERROR : Import failed: {error}")
                        break
                    else:
                        raise
            
            if elapsed_time >= max_wait_time:
                print(f"WARNING : Import timed out after {max_wait_time}s. Check task {task_id} status later.")
                error = f"Import timed out after {max_wait_time}s"
                
        except Exception as e:
            print(f"ERROR : Failed to upload file: {str(e)}")
            error = str(e)
    if not atLeastOne:
        notes = b""
        while not queueResponse.empty():
            q = queueResponse.get(block=False)
            if q:
                if isinstance(q, str):
                    notes += q.encode()
        for tool_iid in tools_iids:
            apiclient.setToolStatus(tool_iid, ["error"], error+"\nSTDOUT:\n"+notes.decode())
    return outputFilePath



import subprocess
import pollenisatorgui.core.components.utils as utils
import psutil
from pollenisatorgui.core.components.apiclient import APIClient
import os

from pollenisatorgui.scripts.lan.utils import checkPath, getNICs


def main(apiclient, appli, **kwargs):
   res, path = checkPath(["responder", "Responder.py", "responder.py"])
   if res is None:
      return False, path
   responder_path = path
   APIClient.setInstance(apiclient)
   responder_conf = ""
   if utils.which_expand_alias("locate"):
        resp = subprocess.run("locate Responder.conf", capture_output=True, text=True, shell=True)
        stdout = resp.stdout
        if stdout.strip() == "":
            if appli:
                import tkinter as tk
                file = tk.filedialog.askopenfilename(title="Locate responder conf file please:", filetypes=[('Config Files', '*.conf')])
                if file:
                    responder_conf = file
                else:
                    return False, "Responder conf not given"
            else:
                responder_conf = input("Responder conf not found, give full path to it please:")
                if responder_conf.strip() == "":
                    return False, "Responder conf not given"
            if not os.path.isfile(responder_conf):
                return False, "Responder conf not found"
        else:
            if appli:
                from pollenisatorgui.core.application.dialogs.ChildDialogCombo import ChildDialogCombo
                dialog = ChildDialogCombo(None, stdout.split("\n"), displayMsg="Choose your responder config file", width=200)
                dialog.app.wait_window(dialog.app)
                if dialog.rvalue is not None:
                    responder_conf = dialog.rvalue.strip()
                    if os.geteuid() != 0:
                        cmd = "sudo "+cmd
                    cmd = 'sed -i -E "s/(HTTP|SMB) = On/\\1 = Off/gm" '+str(responder_conf)
                    appli.launch_in_terminal(None, "sed for responder", cmd, use_pollex=False)
            else:
                possibilites = [x.strip() for x in stdout.split("\n") if x.strip() != ""]
                if len(possibilites) > 1:
                    print("Many responder conf found, choose one :")

                    for i, path in enumerate(possibilites):
                        if path.strip() == "":
                            continue
                        print(str(i+1)+". "+path)
                    responder_conf = input("Choose your responder config file by its number:")
                    try:
                        responder_conf = possibilites[int(responder_conf)-1]
                    except:
                        return False, "Wrong number given"
                else:
                    responder_conf = possibilites[0]
                if responder_conf.strip() == "":
                    return False, "Responder conf not given"
                if os.geteuid() != 0:
                    cmd = "sudo "+cmd
                cmd = 'sed -i -E "s/(HTTP|SMB) = On/\\1 = Off/gm" '+str(responder_conf)
                os.popen(cmd)
   device = getNICs(appli is not None)
   if device is not None and device != "":
      cmd = f"{responder_path} -I {dialog.rvalue} -w -d"
      if os.geteuid() != 0:
        cmd = "sudo "+cmd
        if appli:
            appli.launch_in_terminal(kwargs.get("default_target",None), "Responder listening", cmd, use_pollex=False)
        else:
            subprocess.run(f"{cmd}", shell=True)
   return True, f"Listening responder open"


import os
import uuid
import socket
import sys
from pollenisatorgui.core.components.terminalworker import TerminalWorker
import pollenisatorgui.core.components.utils as utils
import json

def pollterminal():
    """Starts a worker that receives scan orders from the server and upload results
    """
    force_reconnect = False
    if len(sys.argv) > 2:
        print("Usage : pollterminal [--reconnect]")
        sys.exit(1)
    elif len(sys.argv) == 2:
        if sys.argv[1] == "--reconnect":
            print("Reconnecting to server")
            force_reconnect = True
        else:
            print("Invalid option : "+sys.argv[1]+"\nUsage : pollterminal [--reconnect]")
            sys.exit(1)
    verbose = False
    local_settings = utils.load_local_settings()
    sm = TerminalWorker(local_settings)
    myname = os.getenv('POLLENISATOR_WORKER_NAME', str(uuid.uuid4())+"@"+socket.gethostname())
    plugins = list(set(local_settings.get("my_commands",{}).keys()))
    
    sm.connect(myname, plugins, force_reconnect=force_reconnect)
    
    sm.wait()



import json
from datetime import datetime
from pprint import pprint
from bson import ObjectId

from pollenisatorgui.core.components.apiclient import APIClient
from pollenisatorgui.pollenisator import consoleConnect


def nucleiresults(apiclient):
    tags_data = apiclient.findInDb(apiclient.getCurrentPentest(), "tags", { "tags.name": "used-nuclei" }, multi=True)
    ip_ids = []
    if tags_data is not None:
        for tag in tags_data:
            if tag.get("item_type","") == "ips":
                ip_ids.append(tag.get("item_id",""))
    if not ip_ids:  # No IPs tagged with "used-nuclei"
        return []
    list_of_ips = apiclient.findInDb(apiclient.getCurrentPentest(), "ips", { "_id": { "$in": ip_ids } }, multi=True)
    if list_of_ips is None:
        return []
    headerVulnerabilities = []
    for ip in list_of_ips:
        ip_infos = ip.get("infos", {})
        if ip_infos.get("plugin", "").lower() not in ["nuclei", "nuclei.py"]:
            continue
        for finding in ip_infos.get("findings", {}):
            if finding["template-id"] == "http-missing-security-headers":
                headerVulnerabilities.append({
                    "ip": finding.get("ip", ""),
                    "port": finding.get("port", ""),
                    "host": finding.get("host", ""),
                    "header": finding.get("matcher-name", "")
                })
    let ip_objects = ips.data;
    
    return {"vulnTable": vulnDataList, "allDataForExcel": completeData, "newTLSDefectForReport": newTLSDefect}

def updateTLSDefect(defectTLS, new_defect, defect_to_keep, fixes_to_keep):
    defectTLS_description = defectTLS[0]["description"]
    if new_defect.get("description", "") == "":
        new_defect["description"] = defectTLS_description.split("\r\n\r\n")[0] + "\r\n\r\n"
    lignes = defectTLS_description.split("\r\n\r\n")
    title = ""
    for ligne in lignes:
        if ligne.startswith("#"):
            title = ligne.strip("#").strip()
            if title == defect_to_keep:
                new_defect["description"] += ligne + "\r\n\r\n"
                continue
        if title == defect_to_keep:
            new_defect["description"] += ligne + "\r\n\r\n"
            continue
    fixes_to_keep = fixes_to_keep.split(",")
    defectTLS_fixes = defectTLS[0]["fixes"]
    for fix in defectTLS_fixes:
        if str(fix["id"]) in fixes_to_keep and not fix in new_defect["fixes"]:
            new_defect["fixes"].append(fix)
    return new_defect
    
def generateDefect(apiclient, newTLSDefect):
    # Function to generate a defect in the report
    # This function should be implemented to handle the defect creation logic
    print("Generating defect with data:", newTLSDefect)
    defectTLS = apiclient.searchDefect("Défauts d'implémentation du TLS")[0]
    new_TLS_defect = {}
    new_TLS_defect["description"] = ""
    new_TLS_defect["fixes"] = []
    for d in newTLSDefect:
        new_TLS_defect = updateTLSDefect(defectTLS, new_TLS_defect, d[0], d[1])
    defectTLS[0]["description"] = new_TLS_defect["description"]
    defectTLS[0]["fixes"] = new_TLS_defect["fixes"]
    new_defect = defectTLS[0]
    all_defects = apiclient.getDefectTable()
    for d in all_defects:
        if d["title"] == new_defect["title"]:
            print("Defect already exists, updating it.")
            if "_id" in new_defect:
                del new_defect["_id"]
            apiclient.updateInDb(apiclient.getCurrentPentest(), "defects", {"_id":ObjectId(d["_id"])}, {"$set": new_defect})
            return {"status": "success", "data": new_defect}
    apiclient.insertInDb(apiclient.getCurrentPentest(), "defects", new_defect)
    return {"status": "success", "data": new_defect}


def main(apiclient, *args, **kwargs):
    # Example usage of the parseTestSSL function
    results = nucleiresults(apiclient)
    
apiclient = APIClient.getInstance()  # Assuming APIClient is defined elsewhere
apiclient.tryConnection()
res = apiclient.tryAuth()
if not res:
    consoleConnect()
if apiclient.isConnected() is False or apiclient.getCurrentPentest() == "":
    print("You must be connected to a pentest to run this script.")

main(apiclient)
